@@ -14,10 +14,10 @@
 TRACER_DEFAULT_CHANNEL(misc);
 
 
-int arg_split_chars(char **argv, char *buffer, int max_args, char *split)
+u32 arg_split_chars(char **argv, char *buffer, int max_args, char *split)
 {
 	char *ptr = buffer;
-	int argc = 0;
+	u32 argc = 0;
 	TRACE("going to split '%s', max %d buffer\n", buffer, max_args);
 	
 	/* Still return a (which will be empty) if there are no buffer */
@@ -25,7 +25,10 @@ int arg_split_chars(char **argv, char *buffer, int max_args, char *split)
 
 	while(1) {
 		/* Use up all the characters we don't want */
-		while(strchr(split, *ptr) != NULL && *ptr != 0) *ptr++=0;
+		while(strchr(split, *ptr) != NULL && *ptr != 0) {
+			*ptr=0;
+			ptr++;
+		}
 		
 		if(*ptr == 0) return argc;
 
@@ -63,7 +66,7 @@ int arg_split_chars(char **argv, char *buffer, int max_args, char *split)
 }
 
 
-int arg_split(char **argv, char *buffer, int max_args) 
+u32 arg_split(char **argv, char *buffer, int max_args) 
 {
 	return arg_split_chars(argv, buffer, max_args, " \t\n\r");
 }
