@@ -20,7 +20,7 @@ int Monitor_GO(int argc, char **argv)
 		/* Use the existing PC */
 	} else {
 		/* Take the AddressString, and stuff it into PC */
-		sscanf(argv[1], "%lx", &memory_core.pc);
+		sscanf(argv[1], "%x", &memory_core.pc);
 		/* Also stuff it into the stackframe */
 		Memory_Stor(32,memory_core.a[7]+4,memory_core.pc);
 	}
