@@ -17,8 +17,8 @@ TRACER_DEFAULT_CHANNEL(monitor);
 
 int Monitor_MM(int argc, char **argv)
 {
-	long addr;
-	unsigned long value;
+	s32 addr;
+	u32 value;
 	char command[128];
 	char *cmd;
 	char width;
@@ -26,7 +26,7 @@ int Monitor_MM(int argc, char **argv)
 	if(argc != 2) return Monitor_show_help("mm");
 
 	sscanf(argv[0], "mm.%c", &width);
-	sscanf(argv[1], "%lx", &addr);
+	sscanf(argv[1], "%x", &addr);
 	
 	if(width=='w') 	width=16;
 	else		width=32;
@@ -34,10 +34,10 @@ int Monitor_MM(int argc, char **argv)
 	while(1) {
 		if(width==16) {
 			Memory_RetrWord(&value, addr);
-			printf("%08lx: [%04lx] ", addr, value);
+			printf("%08x: [%04x] ", addr, value);
 		} else {
 			Memory_RetrLongWord(&value, addr);
-			printf("%08lx: [%08lx] ", addr, value);
+			printf("%08x: [%08x] ", addr, value);
 		}
 		/* FIXME: Remove this, and have MM use the main (readlined) 
 		 * prompt below */
@@ -49,12 +49,12 @@ int Monitor_MM(int argc, char **argv)
 			break;
 		else {
 			if(width==16) {
-				short TempS;
+				s16 TempS;
 				sscanf(cmd, "%hx", &TempS);
 				Memory_StorWord(addr, TempS);
 			} else {
-				long TempL;
-				sscanf(cmd, "%lx", &TempL);
+				s32 TempL;
+				sscanf(cmd, "%x", &TempL);
 				Memory_StorLongWord(addr, TempL);
 			}
 		}
