@@ -18,14 +18,14 @@ TRACER_DEFAULT_CHANNEL(monitor);
 int Monitor_RM(int argc, char **argv)
 {
 	unsigned int x;
-	unsigned long value;
+	u32 value;
 	char register_str[8];
 
 	if(argc != 3) return Monitor_show_help("rm");
 	sscanf(argv[1], "%s", register_str);
-	sscanf(argv[2], "%lx", &value);
+	sscanf(argv[2], "%x", &value);
 	
-	printf("%s = %lx\n", register_str, value);
+	printf("%s = %x\n", register_str, value);
 
 	if((register_str[0]=='D') || (register_str[0]=='d')) {
 		if(1 != sscanf(register_str+1, "%u", &x)) return Monitor_show_help("rm");
