@@ -16,14 +16,14 @@ TRACER_DEFAULT_CHANNEL(monitor);
 
 
 /* For counting the number of instructions left to trace */
-long Trace_InstructionCount=0;
+s32 Trace_InstructionCount=0;
 
 int Monitor_TRACE(int argc, char **argv)
 {
-	unsigned long frame;
+	u32 frame;
 
 	if(argc > 1)
-		sscanf(argv[1], "%lx", &Trace_InstructionCount);
+		sscanf(argv[1], "%x", &Trace_InstructionCount);
 	else
 		Trace_InstructionCount = 1;
 
@@ -34,18 +34,18 @@ int Monitor_TRACE(int argc, char **argv)
 	Memory_Stor(32,memory_core.a[7],frame);
 	
 	/* Trace on */
-	SRBits->T=1;
+	memory_core.sr.t = SR_T;
 	/* Continue execution */
 	return 0;
 }
 
-void Monitor_TRACE_Entry(short Vector, char *enter_monitor, char *dump_info)
+void Monitor_TRACE_Entry(s16 Vector, char *enter_monitor, char *dump_info)
 {
-	unsigned long frame;
+	u32 frame;
 	*dump_info=1;
 	Trace_InstructionCount--;
 	if(Trace_InstructionCount == 0) {
-		SRBits->T = 0; /* Stop tracing when the count == 0 */
+		memory_core.sr.t = 0; /* Stop tracing when the count == 0 */
 
 		Memory_Retr(&frame, 32, memory_core.a[7]);
 		frame &= ~0x00008000;
