@@ -20,7 +20,7 @@ void timer_5206_init(void);
 
 /* uart_5206.c */
 void serial_5206_init(void);
-char serial_getch(short port_number);
-void serial_putch(short port_number, char c);
+char serial_getch(s16 port_number);
+void serial_putch(s16 port_number, char c);
 
 
