@@ -19,10 +19,10 @@
 TRACER_DEFAULT_CHANNEL(sim);
 
 
-static struct _sim_register *sim_register_lookup_by_offset(long offset);
+static struct _sim_register *sim_register_lookup_by_offset(s32 offset);
 static struct _sim_register *sim_register_lookup_by_name(char *name);
-static void sim_interrupt_assert(short number, short vector);
-static void sim_interrupt_withdraw(short number);
+static void sim_interrupt_assert(s16 number, s16 vector);
+static void sim_interrupt_withdraw(s16 number);
 struct _sim sim_data;
 
 /* name, offset, width, read, write, resetvalue, description */
@@ -196,7 +196,7 @@ static int sim_register_offset_compare(const void *a, const void *b)
 			((struct _sim_register *)b)->offset  );
 }
 
-static struct _sim_register *sim_register_lookup_by_offset(long offset)
+static struct _sim_register *sim_register_lookup_by_offset(s32 offset)
 {
 	struct _sim_register key;
 
@@ -219,8 +219,8 @@ static struct _sim_register *sim_register_lookup_by_name(char *name)
 
 static void sim_setup(struct _memory_segment *s);
 static void sim_fini(struct _memory_segment *s);
-static char sim_read(struct _memory_segment *s, unsigned long *result, short size, unsigned long offset);
-static char sim_write(struct _memory_segment *s, short size, unsigned long offset, unsigned long value);
+static char sim_read(struct _memory_segment *s, u32 *result, s16 size, u32 offset);
+static char sim_write(struct _memory_segment *s, s16 size, u32 offset, u32 value);
 static void sim_reset(struct _memory_segment *s);
 
 
@@ -252,7 +252,7 @@ static void sim_setup(struct _memory_segment *s)
 
 	/* Register the memory region */
 	if(s->mask && s->mask != 0xFFFFFC00) {
-		printf("warning: sim length must be 0x3FF, not 0x%08lx. reset.\n", 
+		printf("warning: sim length must be 0x3FF, not 0x%08x. reset.\n", 
 				~s->mask);
 	}
 	s->mask = 0xFFFFFC00;
@@ -263,6 +263,8 @@ static void sim_setup(struct _memory_segment *s)
 	s->update = NULL;
 	s->data = malloc(sizeof(struct _sim_5206));
 	memset(s->data, 0, sizeof(struct _sim_5206));
+	TRACE("sim alloced with %d bytes at %p\n", sizeof(struct _sim_5206),
+			s->data);
 
 	if(sim_memory_segment) {
 		printf("A SIM has already been defined!\n");
@@ -296,8 +298,8 @@ static void sim_reset(struct _memory_segment *s)
 	}
 }
 
-static char sim_read(struct _memory_segment *s, unsigned long *result, 
-					short size, unsigned long offset)
+static char sim_read(struct _memory_segment *s, u32 *result, 
+					s16 size, u32 offset)
 {
 	struct _sim_register *reg;
 	struct _sim_5206 *sd = (struct _sim_5206 *)s->data;
@@ -317,21 +319,21 @@ static char sim_read(struct _memory_segment *s, unsigned long *result,
 	TRACE("reg=%s, offset=0x%lx, width=%d\n", reg->name, reg->offset, reg->width);
 
 	if(reg->width == 32)
-		*result = *(unsigned long *)((unsigned long)sd + offset);
+		*result = *(u32 *)((u8 *)sd + offset);
 	else if(reg->width == 16)
-		*result = *(unsigned short *)((unsigned long)sd + offset);
+		*result = *(u16 *)((u8 *)sd + offset);
 	else
-		*result = *(unsigned char *)((unsigned long)sd + offset);
+		*result = *(u8 *)((u8 *)sd + offset);
 
 	TRACE("Returning with result=0x%08lx\n", *result);
 	return 1;
 }
 
 
-static char sim_write(struct _memory_segment *s, short size, 
-			unsigned long offset, unsigned long value)
+static char sim_write(struct _memory_segment *s, s16 size, 
+			u32 offset, u32 value)
 {
-/*	register long *ptr = (unsigned long *)0xdeadbeef;*/
+/*	register s32 *ptr = (u32 *)0xdeadbeef;*/
 	struct _sim_register *reg;
 	struct _sim_5206 *sd = (struct _sim_5206 *)s->data;
 	TRACE("size=%d, offset=0x%08lx, value=0x%08lx\n", size, offset, value);
@@ -342,7 +344,7 @@ static char sim_write(struct _memory_segment *s, short size,
 		return 0;
 	}
 	/* Find a register we can write in, if not this one, maybe the 
-	 * next one as long as the offset is the same */
+	 * next one as s32 as the offset is the same */
 	if(!reg->write) {
 		if( ((reg+1)->write) && ((reg+1)->offset == offset) ) reg++;
 	}
@@ -362,11 +364,11 @@ static char sim_write(struct _memory_segment *s, short size,
 		
 
 	if(reg->width == 32)
-		*(unsigned long *)((unsigned long)sd + offset) = value;
+		*(u32 *)((u8 *)sd + offset) = value;
 	else if(reg->width == 16)
-		*(unsigned short *)((unsigned long)sd + offset) = value;
+		*(u16 *)((u8 *)sd + offset) = value;
 	else
-		*(unsigned char *)((unsigned long)sd + offset) = value;
+		*(u8 *)((u8 *)sd + offset) = value;
 
 	return 1;
 	
@@ -382,12 +384,12 @@ static int interrupts_at_level[8] = { 0,0,0,0,0,0,0,0 };
 
 /* This is given to the core when there's an interrupt, this figures
  * out the vector of the interrupt for the core */
-static unsigned long sim_iack_func(unsigned long interrupt_level)
+static u32 sim_iack_func(u32 interrupt_level)
 {
 	struct _sim_5206 *s = (struct _sim_5206 *)sim_memory_segment->data;
-	unsigned long vector=0;
+	u32 vector=0;
 	int x;
-	short mask = 0x1;
+	s16 mask = 0x1;
 
 	TRACE("called for interrupt_level=%d\n", interrupt_level);
 	/* Find the _pending_ interrupt with level == interrupt_level */
@@ -428,9 +430,9 @@ static unsigned long sim_iack_func(unsigned long interrupt_level)
 	return vector;
 }
 	
-static void sim_interrupt_assert(short number, short vector)
+static void sim_interrupt_assert(s16 number, s16 vector)
 {
-	short mask = (0x1 << number);
+	s16 mask = (0x1 << number);
 	struct _sim_5206 *s = (struct _sim_5206 *)sim_memory_segment->data;
 	int icr_level = ICR_LEVEL(s->ICR[number-1]);
 	
@@ -465,9 +467,9 @@ static void sim_interrupt_assert(short number, short vector)
 	return;
 }
 
-static void sim_interrupt_withdraw(short number)
+static void sim_interrupt_withdraw(s16 number)
 {
-	short mask = (0x1 << number);
+	s16 mask = (0x1 << number);
 	struct _sim_5206 *s = (struct _sim_5206 *)sim_memory_segment->data;
 	int icr_level = ICR_LEVEL(s->ICR[number-1]);
 
