@@ -18,6 +18,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <pthread.h>
 
 #include "coldfire.h"
 
@@ -31,12 +32,12 @@ TRACER_DEFAULT_CHANNEL(serial);
  */
 
 struct _UMR1 {
-	unsigned char RxRTS;
-	unsigned char RxIRQ;
-	unsigned char ER;
-	unsigned char PM;
-	unsigned char PT;
-	unsigned char BC;
+	u8 RxRTS;
+	u8 RxIRQ;
+	u8 ER;
+	u8 PM;
+	u8 PT;
+	u8 BC;
 };
 
 /* UMR2 -- Mode Register 2
@@ -46,10 +47,10 @@ struct _UMR1 {
  * +-------+-----+-----+---------------+
  */
 struct _UMR2 {
-	unsigned char CM;
-	unsigned char TxRTS;
-	unsigned char TxCTS;
-	unsigned char SB;
+	u8 CM;
+	u8 TxRTS;
+	u8 TxCTS;
+	u8 SB;
 };
 
 
@@ -60,14 +61,14 @@ struct _UMR2 {
  * +-----+-----+-----+-----+-----+-----+-----+-----+
  */
 struct _USR {
-	unsigned char RB;
-	unsigned char FE;
-	unsigned char PE;
-	unsigned char OE;
-	unsigned char TxEMP;
-	unsigned char TxRDY;
-	unsigned char FFULL;
-	unsigned char RxRDY;
+	u8 RB;
+	u8 FE;
+	u8 PE;
+	u8 OE;
+	u8 TxEMP;
+	u8 TxRDY;
+	u8 FFULL;
+	u8 RxRDY;
 };
 
 
@@ -78,8 +79,8 @@ struct _USR {
  * +-----------------------+-----------------------+
  */
 struct _UCSR {
-	unsigned char RCS;
-	unsigned char TCS;
+	u8 RCS;
+	u8 TCS;
 };
 
 
@@ -90,9 +91,9 @@ struct _UCSR {
  * +-----+-----------------+-----------+-----------+
  */
 struct _UCR {
-	unsigned char MISC;
-	unsigned char TC;
-	unsigned char RC;
+	u8 MISC;
+	u8 TC;
+	u8 RC;
 };
 
 /* UIPCR - Input Port Change Register 
@@ -102,8 +103,8 @@ struct _UCR {
  * +-----+-----+-----+-----+-----+-----+-----+-----+
  */
 struct _UIPCR {
-	unsigned char COS;
-	unsigned char CTS;
+	u8 COS;
+	u8 CTS;
 };
 
 
@@ -114,10 +115,10 @@ struct _UIPCR {
  * +-----+-----+-----+-----+-----+-----+-----+-----+
  */
 struct _UISR {
-	unsigned char COS;
-	unsigned char DB;
-	unsigned char RxRDY;
-	unsigned char TxRDY;
+	u8 COS;
+	u8 DB;
+	u8 RxRDY;
+	u8 TxRDY;
 };
 
 
@@ -128,10 +129,10 @@ struct _UISR {
  * +-----+-----+-----+-----+-----+-----+-----+-----+
  */
 struct _UIMR {
-	unsigned char COS;
-	unsigned char DB;
-	unsigned char FFULL;
-	unsigned char TxRDY;
+	u8 COS;
+	u8 DB;
+	u8 FFULL;
+	u8 TxRDY;
 };
 
 
@@ -139,17 +140,17 @@ struct _uart_data {
 	/* Read */
 	struct _UMR1 UMR1;
 	struct _UMR2 UMR2;
-	unsigned char UMR_pointer;
+	u8 UMR_pointer;
 	struct _USR USR;
 	char URB[3];	/* Recieve Buffer */
-	unsigned char URB_count;
+	u8 URB_count;
 	struct _UIPCR UIPCR;
 	struct _UISR UISR;
-	unsigned char UBG1;
-	unsigned char UBG2;
-	unsigned char UIVR;
-	char UIP;
-
+	u8 UBG1;
+	u8 UBG2;
+	u8 UIVR;
+//	u8 UIP; /* Don't need this, can figure it out from the fd state */
+	
 	/* Write */
 	struct _UCSR UCSR;
 	struct _UCR UCR; /* We dont' need to save this, but thisi is a good
@@ -158,10 +159,14 @@ struct _uart_data {
 	char UACR;
 	struct _UIMR UIMR;
 
+	/* Connections and threads */
 	char transmitter_enabled;
 	char receiver_enabled;
 	int fd;
-	char fd_connected;
+
+	pthread_t tid;
+	pthread_mutex_t lock;
+	int port;
 
 };
 
@@ -169,10 +174,10 @@ static void serial_setup(struct _memory_segment *s);
 static void serial_fini(struct _memory_segment *s);
 static void serial_reset(struct _memory_segment *s);
 static void serial_update(struct _memory_segment *s);
-static char serial_read(struct _memory_segment *s, unsigned long *result, 
-					short size, unsigned long offset);
-static char serial_write(struct _memory_segment *s, short size, 
-				unsigned long offset, unsigned long value);
+static char serial_read(struct _memory_segment *s, u32 *result, 
+					s16 size, u32 offset);
+static char serial_write(struct _memory_segment *s, s16 size, 
+				u32 offset, u32 value);
 
 
 
@@ -181,15 +186,93 @@ void serial_5206_init(void)
 	memory_module_register("uart_5206", &serial_setup);
 }
 
-static unsigned long *default_uart_base_reg = NULL;
-static unsigned long default_uart_base = 0;
+static void *serial_thread(void *arg)
+{
+	struct _memory_segment *s = arg;
+	struct _uart_data *uart = s->data;
+	int status;
+	int bind_fd;
+
+	TRACE("Serial Thread for %s starting...\n", s->name);
+		
+	/* Setup the socket and wait */
+	status = network_setup_on_port(&bind_fd, uart->port);
+
+	while(1) {
+
+		/* We're not clear to send */
+		uart->UIPCR.CTS = 1;
+		uart->UIPCR.COS = 0;
+
+		/* Block on Accept */
+		status = network_accept(&bind_fd, &uart->fd);
+		
+		/* fd_connected */
+		send(uart->fd, s->name, 5, 0);
+		send(uart->fd, "\r\n", 2, 0);
+
+		uart->UIPCR.CTS = 0;
+		uart->UIPCR.COS = 1;
+
+		while(1) {
+			u8 c;
+			status = read(uart->fd, &c, 1);
+
+			/* If status == 0, that's EOF */
+			
+			if(status <= 0) break;
+//			if(status == -1 || c==0) goto interrupt_update;
+			if(c==0) continue; //goto interrupt_update;
+			if(c==0xff) {
+				/* Escape sequence */
+				status = recv(uart->fd, &c, 1, 0);
+				status = recv(uart->fd, &c, 1, 0);
+				continue;
+//				goto interrupt_update;
+			}
+
+			TRACE("%s: Got character %d[%c]\n", s->name, c, c);
+	
+			pthread_mutex_lock(&uart->lock);
+			/* If URB has room, store the character */
+			if(uart->URB_count < 3) {
+				uart->URB[(int)uart->URB_count]=c;
+				uart->URB_count++;
+			} else {
+				ERR("%s: URB_count just overflowed!\n", s->name);
+			}
+
+			/* There's something in the buffer now */
+			uart->USR.RxRDY = 1;
+			if(uart->URB_count == 3) uart->USR.FFULL=1;
+
+			if(uart->UMR1.RxIRQ==0)
+				uart->UISR.RxRDY = uart->USR.RxRDY;
+			else
+				uart->UISR.RxRDY = uart->USR.FFULL;
+			pthread_mutex_unlock(&uart->lock);
+			
+		}
+		
+		close(uart->fd);
+		uart->fd = 0;
+	}
+		
+	return NULL;
+}
+
+
+static u32 *default_uart_base_reg = NULL;
+static u32 default_uart_base = 0;
+static int default_port = 5206;
+
 static void serial_setup(struct _memory_segment *s)
 {
 	struct _uart_data *uart;
-	int port=5206;
+	int res;
 	
 	if(s->mask != 0 && s->mask != 0xFFFFFFC0) {
-		printf("warning: %s length must be 0x3F, not 0x%08lx. reset.\n",
+		printf("warning: %s length must be 0x3F, not 0x%08x. reset.\n",
 				s->name, ~s->mask);
 	}
 	s->mask = 0xFFFFFFC0;
@@ -214,11 +297,15 @@ static void serial_setup(struct _memory_segment *s)
 	uart = malloc(sizeof(struct _uart_data));
 	memset(uart, 0, sizeof(struct _uart_data));
 	s->data = uart;
+
+	uart->port = default_port++;
+	uart->fd = 0;
+	printf("(on port %d)", uart->port);
 	
 	/* Setup a bind for the port */
-	uart->fd_connected = 0;
-	while(-1 == network_setup_on_port(&uart->fd, port)) port++;
-	printf("(on port %d)", port);
+//	uart->fd_connected = 0;
+//	while(-1 == network_setup_on_port(&uart->fd, port)) port++;
+//	printf("(on port %d)", port);
 
 	/* Ensure we have a default serial segment, it will be the one
 	 *  marked with code=xx, or the first segment we find. */
@@ -226,6 +313,13 @@ static void serial_setup(struct _memory_segment *s)
 		default_uart_base_reg = s->base_register;
 		default_uart_base = s->base;
 	}
+
+	pthread_mutex_init(&uart->lock, NULL);
+
+	
+	/* Start the thread */
+	res = pthread_create(&uart->tid, NULL, &serial_thread, s);
+	
 }
 
 static void serial_fini(struct _memory_segment *s)
@@ -254,24 +348,17 @@ static void serial_update(struct _memory_segment *s)
 	struct _uart_data *uart = (struct _uart_data *)s->data;
 	int status;
 	
-/*	short Baud = (uart->UBG1 << 8) | uart->UBG2;*/
+/*	s16 Baud = (uart->UBG1 << 8) | uart->UBG2;*/
 
 /*	TRACE("%s: update: connected=%d, transmitterenabled=%d, receiverenabled=%d\n",
 		PortNumber, uart->fd_connected, uart->transmitter_enabled, uart->receiver_enabled);
 */
-	if(!uart->fd_connected) {
-		/* This eats up a bit of extra time, but we need to do
-		 * a !connected check anyway.. (if we don't.. then below,
-		 * we'll try to read/write to a bad fd).. So, I guess
-		 * accepting connections is most naturally put here */
-		if(network_check_accept(&uart->fd)) {
-			/* fd_connected */
-			send(uart->fd, s->name, 5, 0);
-			send(uart->fd, "\r\n", 2, 0);
-			uart->fd_connected=1;
-		}
+	if(!uart->fd) {
+		/* If there is no connection, there's no chance we can
+		 * do anything with this port */
+		return;
 	}
-
+	
 	/* Transmit stuff */
 	if(uart->transmitter_enabled) {
 		/* If TransmitterReady (TxRDY bit of Status Reg USR)
@@ -282,7 +369,7 @@ static void serial_update(struct _memory_segment *s)
 			 * for us, so we don't need to shift anything
 			 * into transmit buffers */ 
 			TRACE("%s: Sending character %c(%d)\n", s->name, uart->UTB,uart->UTB);
-			if(uart->fd_connected) send(uart->fd, &uart->UTB, 1, 0);
+			if(uart->fd) send(uart->fd, &uart->UTB, 1, 0);
 			/* Signal that there is room in the buffer */
 			uart->USR.TxRDY = 1;
 			/* The UISR duplicates the USR for TxRDY */
@@ -295,47 +382,8 @@ static void serial_update(struct _memory_segment *s)
 	}
 
 	
-	/* Receive stuff */
-	if(uart->receiver_enabled) {
-		unsigned char c;
-		/* See if there is any data */
-/*		TRACE("%s: recv...\n", PortNumber);*/
-		if(uart->fd_connected) {
-			status = recv(uart->fd, &c, 1, 0);
-		} else {
-			status=0;
-			c=0;
-		}
-		if(status == -1 || c==0) goto interrupt_update;
-		if(c==0) goto interrupt_update;
-		if(c==0xff) {
-			/* Escape sequence */
-			status = recv(uart->fd, &c, 1, 0);
-			status = recv(uart->fd, &c, 1, 0);
-			goto interrupt_update;
-		}
-
-		TRACE("%s: Got character %d[%c]\n", s->name, c, c);
-
-		/* If URB has room, store the character */
-		if(uart->URB_count < 3) {
-			uart->URB[(int)uart->URB_count]=c;
-			uart->URB_count++;
-		} else {
-			ERR("%s: URB_count just overflowed!\n", s->name);
-		}
-
-		uart->USR.RxRDY = 1;
-		if(uart->URB_count == 3) uart->USR.FFULL=1;
-
-		if(uart->UMR1.RxIRQ==0)
-			uart->UISR.RxRDY = uart->USR.RxRDY;
-		else
-			uart->UISR.RxRDY = uart->USR.FFULL;
-	}
-
 interrupt_update:
-
+	pthread_mutex_lock(&uart->lock);
 	/* Adjust the interrupt status */
 	/* If the TX is masked on, and there is room in the buffer, we 
 		want the interrupt on */
@@ -363,11 +411,13 @@ interrupt_update:
 		the interrupts on , so turn 'em off */
 /*	TRACE("%s: No interrupt conditions, withdrawing any interrupt requests\n", s->name);*/
 	sim->interrupt_withdraw(s->interrupt_line);
+	pthread_mutex_unlock(&uart->lock);
 	return;
 
 interrupts_on:
 	TRACE("%s: Posting interrupt request for %s\n", s->name);
 	sim->interrupt_assert(s->interrupt_line, uart->UIVR);
+	pthread_mutex_unlock(&uart->lock);
 	return;
 	
 
@@ -375,8 +425,8 @@ interrupts_on:
 
 
 /* Handles hardware writes to the serial ports */
-static char serial_write(struct _memory_segment *s, short size, 
-				unsigned long offset, unsigned long value)
+static char serial_write(struct _memory_segment *s, s16 size, 
+				u32 offset, u32 value)
 {
 	struct _uart_data *uart = (struct _uart_data *)s->data;
 
@@ -384,6 +434,7 @@ static char serial_write(struct _memory_segment *s, short size,
 		and that MBAR has already been subtracted from it */
 
 	TRACE("%s: size=%d, offset=0x%04lx, value=0x%08lx\n", s->name, size, offset, value);
+	pthread_mutex_lock(&uart->lock);
 
 	switch(offset) {
 	case 0x0000: /* Mode Register (UMR1, UMR2) */
@@ -574,17 +625,19 @@ static char serial_write(struct _memory_segment *s, short size,
 		TRACE("%s: Reset Output Port Bit (UOP0) (no effect)\n", s->name);
 		break;
 	default:
+		pthread_mutex_unlock(&uart->lock);
 		return 0;
 
 	}
+	pthread_mutex_unlock(&uart->lock);
 	return 1;
 	
 }
 
 
 
-static char serial_read(struct _memory_segment *s, unsigned long *result, 
-					short size, unsigned long offset)
+static char serial_read(struct _memory_segment *s, u32 *result, 
+					s16 size, u32 offset)
 {
 	struct _uart_data *uart = (struct _uart_data *)s->data;
 	/* We ASSUME that we are passed an offset that is valid for us, 
@@ -594,6 +647,8 @@ static char serial_read(struct _memory_segment *s, unsigned long *result,
 
 	*result=0;
 
+	pthread_mutex_lock(&uart->lock);
+	
 	switch(offset) {
 	case 0x0000: /* Mode Register (UMR1, UMR2) */
 		break;
@@ -645,13 +700,17 @@ static char serial_read(struct _memory_segment *s, unsigned long *result,
 		break;
 
 	case 0x0010: /* Input Port Change Register (UIPCR) */
+		*result = (uart->UIPCR.COS ? 0x40 : 0x00) |
+			  (uart->UIPCR.CTS ? 0x01 : 0x00);
+		/* Reading from this register should clear the 
+		 * COS (change of state) if it was asserted */
+		uart->UIPCR.COS = 0;
 		break;
 	case 0x0014: /* Interrupt Status Register (UISR) */
-		*result = 
-			(uart->UISR.COS  ? 0x80 : 0x00) |
-			(uart->UISR.DB   ? 0x04 : 0x00) |
-			(uart->UISR.RxRDY? 0x02 : 0x00) |
-			(uart->UISR.TxRDY? 0x01 : 0x00);
+		*result = (uart->UISR.COS  ? 0x80 : 0x00) |
+			  (uart->UISR.DB   ? 0x04 : 0x00) |
+			  (uart->UISR.RxRDY? 0x02 : 0x00) |
+			  (uart->UISR.TxRDY? 0x01 : 0x00);
 		break;
 	case 0x018: /* Baud Rate Generator Prescale MSB (UBG1) */
 		*result = uart->UBG1;
@@ -663,15 +722,19 @@ static char serial_read(struct _memory_segment *s, unsigned long *result,
 		*result = uart->UIVR;
 		break;
 	case 0x0034: /* Input Port Register (UIP) */
-		*result = 0;
+		/* Set to 0 (meaning nCTS=0, meaning we're Clear to Send) 
+		 * if there is something connected */
+		*result = (uart->fd != 0) ? 0 : 1;
 		break;
 	case 0x0038: /* NO NOT ACCESS */
 	case 0x003C: /* NO NOT ACCESS */
 	default: 
 		ERR("%s: Unaligned access!\n",s->name);
+		pthread_mutex_unlock(&uart->lock);
 		return 0;
 
 	}
+	pthread_mutex_unlock(&uart->lock);
 	TRACE("%s: result=0x%08lx\n", s->name, *result);
 	return 1;
 	
@@ -681,11 +744,11 @@ static char serial_read(struct _memory_segment *s, unsigned long *result,
 /* FIXME: We need to fix the Serial_getch and Serial_putch routines, 
  * so they use the serial port access rouintes above... The routines
  * below are for TRAP #15 */
-char serial_getch(short port_number)
+char serial_getch(s16 port_number)
 {
-	unsigned long base = *default_uart_base_reg + default_uart_base;
-	unsigned long usr;
-	unsigned long c;
+	u32 base = *default_uart_base_reg + default_uart_base;
+	u32 usr;
+	u32 c;
 	
 	/* Enable Receiver */
 	Memory_StorByte(base + 0x08, 0x05);
@@ -704,10 +767,10 @@ char serial_getch(short port_number)
 	return (char)c;
 }
 
-void serial_putch(short port_number, char c)
+void serial_putch(s16 port_number, char c)
 {
-	unsigned long base = *default_uart_base_reg + default_uart_base;
-	unsigned long usr;
+	u32 base = *default_uart_base_reg + default_uart_base;
+	u32 usr;
 
 	/* Enable transmitter */
 	Memory_StorByte(base + 0x08, 0x05);
