@@ -5,11 +5,11 @@
 #define PPRO 1
 #define CPU_SPEED 600000000 
 
-unsigned long long Profile_time_in_ms(void)
+u32 s32 Profile_time_in_ms(void)
 {
-	unsigned long timer_hi;
-	unsigned long timer_lo;
-	unsigned long long result=0;
+	u32 timer_hi;
+	u32 timer_lo;
+	u32 s32 result=0;
 
 #ifdef PPRO
 	asm volatile ("cpuid"
@@ -67,10 +67,10 @@ static char *eng(double value, int places)
 }
 
 
-void Profile_MakeTimeString(char *Buffer, unsigned long long Low, unsigned long long High)
+void Profile_MakeTimeString(char *Buffer, u32 s32 Low, u32 s32 High)
 {
 	double ExecTime;
-	unsigned long long Elapsed=0;
+	u32 s32 Elapsed=0;
 	Elapsed=(abs((High-Low)));
 	ExecTime=(double) (((double)Elapsed)/(double)CPU_SPEED);
         sprintf(Buffer, "%s", eng(ExecTime,6));
