@@ -25,7 +25,7 @@ int Stats_Build_EA (int Register, int Mode)
 	case 6:	Stats_Mode[sARIIB]++; break;	/* (d8,An,Xi) */
 	case 7: switch (Register) {
 		case 0:	Stats_Mode[sAS]++; break; /* word addressing */
-		case 1:	Stats_Mode[sAL]++; break; /* long addressing */
+		case 1:	Stats_Mode[sAL]++; break; /* s32 addressing */
 		case 2:	Stats_Mode[sPCID]++; break; /* (d16,PC) */
 		case 3:	Stats_Mode[sPCIIB]++; break; /* (d8,PC,Xi) */
 		case 4:	Stats_Mode[sIM]++; break; /* (d8,PC,Xi) */
