@@ -10,8 +10,8 @@
       -(Ay)         100    Aregnum     ari with predecrement     
      (d16,Ay)       101    Aregnum     ari with displacement 
     (d8,Ay,Xi)      110    Aregnum     ari with index base
-     (xxx).W        111    000         absolute short
-     (xxx).L        111    001         absolute long
+     (xxx).W        111    000         absolute s16
+     (xxx).L        111    001         absolute s32
      #<data>        111    100         immediate  
      (d16,PC)       111    010         program counter indirect disp
     (d8,PC,Xi)      111    011         pci with index base
